  --
  -- Developer : Andrew Tunnecliffe 
  -- Contact: andrew.207@gmail.com http://www.atunnecliffe.com
  -- Date : Jan-2014
  -- All Code (C) 2014 Andrew Tunnecliffe
  --
 
 RESPAWN_TIME = 3
 SCORE = {}
 
function love.load()
    pause = false;
    debug = false;
    gravityFlag = true;
    g = love.graphics
    playerColor = {255,0,128}
    groundColor = {25,200,25}
    blockColor = {255,255,0}
    CLIMBING_ABILITY = 5
    GROUND_HEIGHT = HEIGHT*SCALE
    
    -- instantiate our player and set initial values
    require "player"
    require "platform"
    require "camera"
    require "block"
    require "enemy"
    
    players = {}
    players[1] = player:new(10, 10)
    players[2] = player:new(10, 10)
    cam = camera:new(-WIDTH*SCALE/4, -WIDTH*SCALE+WIDTH*SCALE/4)    
    
    enemies = {}
    --enemies[1] = enemy:new(10, 10)
    
    
    for i,d in ipairs(players) do
      d.x = 290
      d.y = 0
      d.width = 12
      d.height = 20
      d.jumpSpeed = -450
      d.runSpeed = 100
      d.index = i
      cam:addPlayer(d)
      table.insert(SCORE, 0)
    end
    
    players[2].bot = true
    players[2].target = players[1]
    players[2].x = 390
    
    for i,v in ipairs(players) do
      v.current["above"..i] = 0
      v.current["below"..i] = 0
      v.current["left"..i] = 0
      v.current["right"..i] = 0
      v.last["above"..i] = 0
      v.last["below"..i] = 0
      v.last["left"..i] = 0
      v.last["right"..i] = 0
    end
    
    gravity = 1400
    
    platforms = {}
    platforms[1] = platform:new(-250, 250, 4000, 50)
    platforms[2] = platform:new(100, 200, 50, 10)
    platforms[3] = platform:new(210, 170, 50, 10)
    platforms[4] = platform:new(350, 175, 30, 10)
    platforms[5] = platform:new(275, 160, 30, 10)
    platforms[6] = platform:new(450, 150, 30, 10)
    platforms[7] = platform:new(530, 100, 80, 10)
    platforms[8] = platform:new(550, 200, 100, 10)
    platforms[9] = platform:new(125, 60, 200, 10)
    platforms[10] = platform:new(375, 70, 100, 10)
    platforms[11] = platform:new(390, 190, 50, 10)
    platforms[12] = platform:new(490, 190, 50, 10)
    platforms[13] = platform:new(0, 150, 20, 10)
    
    for _,v in ipairs(platforms) do 
      for i,d in ipairs(players) do
        v.current["above"..i] = 0
        v.current["below"..i] = 0
        v.current["left"..i] = 0
        v.current["right"..i] = 0
        v.last["above"..i] = 0
        v.last["below"..i] = 0
        v.last["left"..i] = 0
        v.last["right"..i] = 0
        v.activatedLeft = false
        v.activatedRight = false
        v.activatedTop = false
        v.activatedBottom = false
      end
    end
    
    blocks = {}
    blocks[1] = block:new(700, 175, 25, 25)
    blocks[2] = block:new(750, 175, 25, 25)
    blocks[3] = block:new(800, 175, 25, 25)
    blocks[4] = block:new(750, 100, 25, 25)
    
    for _,v in ipairs(blocks) do 
      v.current["above"] = 0
      v.current["below"] = 0
      v.current["left"] = 0
      v.current["right"] = 0
      v.last["above"] = 0
      v.last["below"] = 0
      v.last["left"] = 0
      v.last["right"] = 0
    end
end

function love.checkForPlayerCollisions(players)
  for i,v in ipairs(players) do
    for j,z in ipairs(players) do
      if z.respawnTime < 1 then
        v.last["above"..z.index] = v.current["above"..z.index]
        v.last["below"..z.index] = v.current["below"..z.index]
        v.last["left"..z.index] = v.current["left"..z.index]
        v.last["right"..z.index] = v.current["right"..z.index]
        v.current["above"..z.index] = 0
        v.current["below"..z.index] = 0
        v.current["left"..z.index] = 0
        v.current["right"..z.index] = 0
        
        if z:getBottomEdge() < v:getTopEdge() then v.current["above"..z.index] = 1 end -- we are above
        if z:getRightEdge() < v:getLeftEdge() then v.current["left"..z.index] = 1 end -- we are left
        if z:getLeftEdge() > v:getRightEdge() then v.current["right"..z.index] = 1 end -- we are right
        if z:getTopEdge() > v:getBottomEdge() then v.current["below"..z.index] = 1 end -- we are below
          
        -- -- -- -- -- -- -- -- -- -- -- 
        -- Top
        -- -- -- -- -- -- -- -- -- -- -- 
        if v.last["above"..z.index] == 1 and v.current["above"..z.index] == 0 and v.current["left"..z.index] == 0 and v.current["right"..z.index] == 0 and v.respawnTime < 0.01 then 
          v.height = 10
          v.onFloor = false
          v.respawnTime = RESPAWN_TIME
          SCORE[j] = SCORE[j] + 1
          if v.ySpeed < 0 then v.ySpeed = 0 end
          z.ySpeed = z.ySpeed / -2
        end
      end
    end
  end
end
 
function love.update(dt)
  love.checkForPlayerCollisions(players)
  if pause == false then
    if love.keyboard.isDown("right") then
      players[1]:moveRight()
      cam:movingRight()
    end
    if love.keyboard.isDown("left") then
      players[1]:moveLeft()
      cam:movingLeft()
    end
    if love.keyboard.isDown("left") then
      --players[1]:dropThroughFloor()
    end
    
    if debug then
      if not love.keyboard.isDown("z") then
        if love.keyboard.isDown("d") then
           players[1].x = players[1].x + 1
        end
        if love.keyboard.isDown("a") then
            players[1].x = players[1].x - 1
        end
        if love.keyboard.isDown("w") then
            players[1].y = players[1].y - 1
        end
        if love.keyboard.isDown("s") then
            players[1].y = players[1].y + 1
        end
      end
    end
  
    -- if the x key is pressed...
    if love.keyboard.isDown("x") then
    -- make the player jump
      players[1]:jump()
    end
    if love.keyboard.isDown("z") then
    -- make the player jump
      players[2]:jump()
    end
   
    -- update the player's position
    for i,v in ipairs(players) do
      v:update(dt, gravity, platforms)
    end
    cam:update(dt)
   
    -- stop the player when they hit the borders
    for i,d in ipairs(players) do d.collided = false; end
    
    for i,d in ipairs(players) do
      for _,v in ipairs(platforms) do 
        d:compareLocation(v, dt)
      end
      for _,v in ipairs(blocks) do 
        d:compareLocation(v, dt)
      end
    end
  end
end
 
function love.draw()
  love.graphics.translate(cam.x, cam.y)
  --love.graphics.scale(2,2)
  -- draw the player shape
  for i,v in ipairs(players) do
    g.setColor(playerColor)
    v:draw()
  end
  
  -- draw the ground
  g.setColor(groundColor)
  for _,v in ipairs(platforms) do 
    v:draw()
  end
  
  -- draw blocks
  g.setColor(blockColor)
  for _,d in ipairs(blocks) do 
    d:draw()
  end
  
  -- draw scoreboard
  g.print("SCORES\n", -cam.x + (WIDTH*SCALE)/2, -cam.y)
  for i,d in ipairs(players) do
    g.print("Player "..i..": "..SCORE[i], -cam.x + (WIDTH*SCALE)/2, -cam.y + 15*i)
  end
  
  -- debug information
  if (debug) then
    g.setColor(255, 255, 255)
    local isTrue = ""
    g.print("Player coordinates: ("..math.floor(players[1].x).." - "..math.floor(players[1].x+players[1].width)..", "..math.floor(players[1].y).." - "..math.floor(players[1].y+players[1].height)..")", -cam.x, -cam.y + 5)
    g.print("Current state: "..players[1].state, -cam.x, -cam.y + 20)
    g.print("Speeds: "..players[1].xSpeed..", "..players[1].ySpeed, -cam.x, -cam.y + 35)
    g.print("Player notice: "..players[1].notice, -cam.x, -cam.y + 50)
    a = ""
    if blocks[1].bumped == true then a = "Block 1 bumped: True" else a = "Block 1 bumped: False" end
    g.print(a, -cam.x, -cam.y + 80)
  end  
end
 
function love.keyreleased(key)
  if key == "escape" then
    love.event.push("kill")   
  end
  if (key == "right") or (key == "left") then
    players[1].decelarating = true;
  end
  
  if (key == "`") then
    debug = not debug
  end
  if (key == "tab") then
    gravityFlag = not gravityFlag
  end
  if (key == "q") then
    players[1].x = 10 players[1].y = 10
  end
  if (key == "p") then
    pause = not pause
  end
  -- debug movement
  if love.keyboard.isDown("z") then
    if key == "d" then
      players[1].x = players[1].x + 1
    end
    if key == "a" then
      players[1].x = players[1].x - 1
    end
    if key == "w" then
      players[1].y = players[1].y - 1
    end
    if key == "s" then
      players[1].y = players[1].y + 1
    end
  end    
end