  --
  -- Developer : Andrew Tunnecliffe 
  -- Contact: andrew.207@gmail.com http://www.atunnecliffe.com
  -- Date : Jan-2014
  -- All Code (C) 2014 Andrew Tunnecliffe
  --

player = {} 
 
function player:new(x, y)
  local object = {
    x = x,
    y = y,
    accelaration = 25,
    xSpeedMax = 200,
    ySpeedMax = 800,
    lastXOnGround = x,
    lastYOnGround = y,
    width = 0,
    height = 0,
    xSpeed = 0,
    ySpeed = 0,
    state = "",
    last = {},
    current = {},
    jumpSpeed = 0,
    runSpeed = 0,
    canJump = false,
    jumpCooldown = 0,
    onFloor = false,
    dead = false,
    standingOn = 0,
    targetPlatform = 0,
    lastStandingOn = 0,
    decelarating = false,
    collided = false,
    collidedLeft = false,
    colidedRight = false,
    respawnTime = 0,
    colorMod = 0,
    jumpHeight = -60,
    jumpDistance = 140,
    notice = "blank",
    platforms = {},
    path = {},
    bot = false,
    botDifficulty = 1,
    messedUp = 0,
    moved = false,
    targeting = false,
    targetx = 0,
    targety = 0,
    onRight = 0,
    onLeft = 0,
    above = 0,
    below = 0;
  }
  setmetatable(object, { __index = player })
  return object
end
 
-- Movement functions
function player:jump()
  if self.canJump then
    self.ySpeed = self.jumpSpeed
    self.canJump = false
    self.jumpCooldown = 99
    self.onFloor = false
    self.standingOn = 0
  end
end
 
function player:moveRight()
  if self.xSpeed < self.xSpeedMax then self.xSpeed = self.xSpeed + self.accelaration end
  self.decelarating = false;
end
 
function player:moveLeft()
  if self.xSpeed > -self.xSpeedMax then self.xSpeed = self.xSpeed + -self.accelaration end
  self.decelarating = false;
end
 
function player:decelarate()
  if (self.xSpeed > 0) then self.xSpeed = self.xSpeed - self.accelaration end
  if (self.xSpeed < 0) then self.xSpeed = self.xSpeed + self.accelaration end
end
 
function player:hitFloor(maxY)
  self.y = maxY - self.height
  self.ySpeed = 0 
  self.jumpCooldown = 0.05
  self.onFloor = true
  self.JUMPINGAT = false
end

function player:dropThroughFloor()
  if self.standingOn ~= 0 then
    self.y = self.standingOn.y + self.standingOn.height
    self.onFloor = false
    self.standingOn = 0
  end
end

function player:hitRoof(maxY)
  self.y = maxY
  self.ySpeed = 0
end

function player:hitWallLeft(maxX)
  self.collidedLeftWall = maxX
  self.x = maxX
  self.xSpeed = 0;
  self.collidedLeft = true;
end

function player:hitWallRight(maxX)
  self.collidedRightWall = maxX - self.width
  self.x = maxX - self.width
  self.xSpeed = 0;
  self.collidedRight = true;
end

function player:compareLocation(c, dt)
  c.last["above"..self.index] = c.current["above"..self.index]
  c.last["below"..self.index] = c.current["below"..self.index]
  c.last["left"..self.index] = c.current["left"..self.index]
  c.last["right"..self.index] = c.current["right"..self.index]
  c.current["above"..self.index] = 0
  c.current["below"..self.index] = 0
  c.current["left"..self.index] = 0
  c.current["right"..self.index] = 0
  
  if self:getBottomEdge() < c:getTopEdge() then c.current["above"..self.index] = 1 end -- we are above
  if self:getRightEdge() < c:getLeftEdge() then c.current["left"..self.index] = 1 end -- we are left
  if self:getLeftEdge() > c:getRightEdge() then c.current["right"..self.index] = 1 end -- we are right
  if self:getTopEdge() > c:getBottomEdge() then c.current["below"..self.index] = 1 end -- we are below
    
  -- -- -- -- -- -- -- -- -- -- -- 
  -- Top
  -- -- -- -- -- -- -- -- -- -- -- 
  if c.last["above"..self.index] == 1 and c.current["above"..self.index] == 0 and c.current["left"..self.index] == 0 and c.current["right"..self.index] == 0 then 
    self:hitFloor(c:getTopEdge()) 
    self.lastStandingOn = self.standingOn 
    self.standingOn = c 
  end
  if self.standingOn == c then -- falling off
    if c.current["left"..self.index] == 1 and c.last["left"..self.index] == 0 then self.onFloor = false self.lastStandingOn = self.standingOn self.standingOn = 0 end 
    if c.current["right"..self.index] == 1 and c.last["right"..self.index] == 0 then self.onFloor = false self.lastStandingOn = self.standingOn self.standingOn = 0 end
  end
  
  -- -- -- -- -- -- -- -- -- -- -- 
  -- Bottom
  -- -- -- -- -- -- -- -- -- -- -- 
  if c.last["below"..self.index] == 1 and c.current["below"..self.index] == 0 and c.current["left"..self.index] == 0 and c.current["right"..self.index] == 0 then if c.isBlock == true then self:hitRoof(c:getBottomEdge()) c.bumped = true end end
  
  -- -- -- -- -- -- -- -- -- -- --
  -- Bumping
  -- -- -- -- -- -- -- -- -- -- --
  if c.isBlock == true then 
    if c.bumped == true then 
      c.yInt = c.y - 50 
      c.bumped = false 
      c.yOrigin = c.y 
      c.ySpeed = -2 
      c.bumping = true 
    end
    if c.bumping == true then c.ySpeed = c.ySpeed + 10 * dt end
    if c.y > c.yOrigin then 
      c.ySpeed = 0
      c.bumping = false
      c.y = c.yOrigin
    end
      
    c.y = c.y + c.ySpeed
  end
  
end

-- returns true if player can jump on given platform from current location
function player:canJumpOn(p)
  local ok = true
  if p.x - self.x < self.jumpDistance and p.x - self.x > -self.jumpDistance then ok = true else ok = false end
  if p.y - self.y > self.jumpHeight and ok == true then ok = true else ok = false end
  return ok
end

-- returns true if player can jump on given platform from arbitrary location
function player:couldJumpOn(x, y, p)
  local ok = true
  --if p.x - x < self.jumpDistance and p.x - x > -self.jumpDistance then ok = true else ok = false end
  if p.y - y > self.jumpHeight and ok == true then ok = true else ok = false end
  return ok
end

-- returns the index of the max (obj.y) value from table
function player:getMaxYFromTable(tab)
  local maxIndex = 1
  local currentMaxY = 999999
  for i,v in ipairs(tab) do
    if v.y < currentMaxY then currentMaxY = v.y maxIndex = i end
  end
  
  return maxIndex
end

-- Update's AI targetted platform
function player:updateTarget(platform)
  if self.x < platform.x then self.onRight = 1 self.onLeft = 0 else 
    self.onLeft = 1 self.onRight = 0 end
  if self.y + self.height < platform.y then self.below = 1 self.above = 0 else
    self.below = 0 self.above = 1 end
  self.targetPlatform = platform
end

-- Update's AI targetted location to the edge of the current platform that is closest to the player
function player:updateTargetArbitrary(platform)
  if self.x < self.target.x then self.onRight = 1 self.onLeft = 0 else 
    self.onLeft = 1 self.onRight = 0 end
  if self.y + self.height < self.target.y then self.below = 1 self.above = 0 else
    self.below = 0 self.above = 1 end
    
    
end

function player:findPathToTarget(platforms)
  -- if we're above, then figure out what platforms are in between. 
  local yBetween = {}
  local yBetween2 = {}
  local count = 0
  -- Go through tables selecting the ones the player can jump on from current position
  for _,v in ipairs(platforms) do
    if self:couldJumpOn(self.lastXOnGround, self.lastYOnGround, v) == true and v:getTopEdge() > self.target:getBottomEdge() then v.doColor = true table.insert(yBetween, v) end
  end
  
  -- Select the highest platform from the list'
  if #yBetween <= 0 then return 1 end
  local index = self:getMaxYFromTable(yBetween)
  yBetween[index].setColor = true
  yBetween[index].colorMod = {255, 255, 0}
  self:updateTarget(yBetween[index])
end

function player:doAIStuff(platforms)
    self.target.notice = ""
    
    if self.colorMod ~= 0 then
      self.colorMod.doColor = true
    end
    
    -- This code will send the AI in the direction of their target. Pathfinding will override this if necessary.
    if self.x < self.target.x then self.onRight = 1 self.onLeft = 0 else 
      self.onLeft = 1 self.onRight = 0 end
    if self.y+self.height < self.target.y then self.below = 1 self.above = 0 else
      self.below = 0 self.above = 1 end
    
    local skip = false
    local goingDown = false

    if self.standingOn == self.target.standingOn then skip = true end -- if on same platform
    if self.standingOn == self.lastStandingOn then skip = true end -- if standing on the same platform as last loop
    if self.y < self.target.y then skip = true goingDown = true end -- if we're above the player
    --if #self.path < 0 then skip = true end
    -- pathing
    if skip == false and self.ySpeed == 0 then -- create path to top
      self:findPathToTarget(platforms)
    end
    if goingDown == true and self.standingOn ~= 0 then
      self:updateTargetArbitrary(self.standingOn:getLeftEdge(), self.y)
    end
    
    if self.standingOn ~= self.targetPlatform and self.targetPlatform ~= 0 then
      self:updateTarget(self.targetPlatform)
    elseif self.standingOn == self.targetPlatform then self.targetPlatform = 0 end
      
    --local fail = love.math.random(1, 5 - self.botDifficulty) == 1
    --if self.messedUp <= 0 then if self.moved == false then if self.onLeft == 1 or self.onRight == 1 then if fail == true then self.onLeft = 0 self.onRight = 0 self.messedUp = 80 end end end end
    
    --if self.messedUp > 0 then self.messedUp = self.messedUp - 1 else
      if self.onRight == 1 then self:moveRight() self.moved = true end
      if self.onLeft == 1 then self:moveLeft() self.moved = true end
      if self.moved == false then self.decelarating = true end
      if self.above == 1 then self:jump() end
    --end
end
 
-- Update function
  function player:update(dt, gravity, platforms) 
  
  -- If we're dead, we need to time the respawn
  if self.respawnTime > 0 and self.onFloor == true then self.respawnTime = self.respawnTime - dt self.dead = true else 
    -- And also reposition and grow big again
    if self.dead == true then self.dead = false self.x = math.random(0, 500) self.y = 30 self.onFloor = false self.height = 20 end
    
    
    if self.bot == true and self.target ~= 0 then
      self:doAIStuff(platforms)
    end
    
    -- Left and right ray blocks
    --if self.collidedRight == true and self.x + (self.xSpeed * dt) > self.collidedRightWall then self.xSpeed = 0 end
    --if self.collidedLeft == true and self.x + (self.xSpeed * dt) < self.collidedLeftWall then self.xSpeed = 0 end
    if self.x < 0 - self.width then self.x = WIDTH*SCALE end
    if self.x > WIDTH*SCALE then self.x = 0 - self.width end
    
    -- Clamp X and Y speeds
    if self.ySpeed > self.ySpeedMax then self.ySpeed = self.ySpeedMax end
    if self.ySpeed < -self.ySpeedMax then self.ySpeed = -self.ySpeedMax end
      
    self.x = self.x + (self.xSpeed * dt)
    self.y = self.y + (self.ySpeed * dt)
      
    -- apply gravity
    if gravityFlag then
      if self.onFloor == false then
        self.ySpeed = self.ySpeed + (gravity * dt)
      end
    end

    -- slow down
    if (self.decelarating) then self:decelarate() end
    if self.jumpCooldown < 0 then self.canJump = true else self.jumpCooldown = self.jumpCooldown - dt end
    
    if self.onFloor == true then
      self.lastXOnGround = self.x
      self.lastYOnGround = self.y
    end
   
    -- update the player's state
    self:updateState()
  end
end

function player:draw()
  if self.bot == true then love.graphics.setColor(255, 0, 0) end
  g.rectangle("fill", self.x, self.y, self.width, self.height)
  if debug then
    g.setColor(255,255,0)
    g.line(self:getRightEdge(), self.y - self.height, self:getRightEdge(), self.y + self.height * 2)
    g.line(self:getLeftEdge(), self.y - self.height, self:getLeftEdge(), self.y + self.height * 2)
    g.line(self.x - self.width, self:getTopEdge(), self.x + self.width * 2, self:getTopEdge())
    g.line(self.x - self.width, self:getBottomEdge(), self.x + self.width * 2, self:getBottomEdge())
  end
end

function player:updateState()
  if not(self.canJump) then
    if self.ySpeed < 0 then
      self.state = "jump"
    elseif self.ySpeed > 0 then
      self.state = "fall"
    end
  else
    if self.xSpeed > 0 then
      self.state = "moveRight"
    elseif self.xSpeed < 0 then
      self.state = "moveLeft"
    else
      self.state = "stand"
    end
  end    
  if self.collidedLeft == true then
    self.state = self.state.."ing on left wall"
    if self.xSpeed > 0 then self.collidedLeft = false end
  end
  if self.collidedRight == true then
    self.state = self.state.."ing on right wall"
    if self.xSpeed < 0 then self.collidedRight = false end
  end
end

function player:getRightEdge() 
 return self.x+self.width  
end

function player:getLeftEdge() 
 return self.x  
end

function player:getTopEdge() 
 return self.y  
end

function player:getBottomEdge() 
 return self.y + self.height  
end